@@ -2,7 +2,7 @@
 #
 # The PyMca X-Ray Fluorescence Toolkit
 #
-# Copyright (c) 2004-2018 European Synchrotron Radiation Facility
+# Copyright (c) 2004-2020 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -245,9 +245,9 @@ class StackBase(object):
         self._stack.info['McaIndex'] = mcaIndex
         self._stack.info['FileIndex'] = fileIndex
         self._stack.info['OtherIndex'] = otherIndex
-        self.stackUpdated()
+        self.stackUpdated(info.get("positioners", None))
 
-    def stackUpdated(self):
+    def stackUpdated(self, positioners=None):
         """
         Recalculates the different images associated to the stack
         """
@@ -330,16 +330,14 @@ class StackBase(object):
                                 self._stack.info["McaLiveTime"].sum()
             else:
                 print("Not an array. Skipping time information")
-                del dataObject.info["McaLiveTime"]
+                #del dataObject.info["McaLiveTime"]
         if not hasattr(self._stack, 'x'):
             self._stack.x = None
         if self._stack.x in [None, []]:
             self._stack.x = [numpy.arange(len(mcaData0)).astype(numpy.float)+\
                                 self._stack.info.get('Channel0', 0.0)]
-            dataObject.x = [self._stack.x[0]]
-        else:
-            # for the time being it can only contain one axis
-            dataObject.x = [self._stack.x[0]]
+        # for the time being it can only contain one axis
+        dataObject.x = [self._stack.x[0]]
 
         dataObject.y = [mcaData0]
 
@@ -366,6 +364,12 @@ class StackBase(object):
                          'to': dataObject.x[0][-1]}
 
         self.updateROIImages()
+        if positioners is not None:
+            try:
+                self.setPositioners(positioners)
+            except:
+                logging.error("Error setting positioners. Ignoring them")
+                self._clearPositioners()
         for key in self.pluginInstanceDict.keys():
             self.pluginInstanceDict[key].stackUpdated()
 
@@ -1044,7 +1048,7 @@ class StackBase(object):
 
         Default output has the form:
             xvalues, yvalues, legend, dict
-            where dict is a dictionnary containing curve info.
+            where dict is a dictionary containing curve info.
             For the time being, only the plot labels associated to the
             curve are warranted to be present under the keys xlabel, ylabel.
 
@@ -1092,7 +1096,7 @@ class StackBase(object):
             is not a scalar, list or numpy array.
         :raise: RuntimeError if any positioner is a list and copy=False
         """
-        if not isinstance(positioners, dict):
+        if not hasattr(positioners, "items"):
             raise TypeError("Dictionary expected for positioners")
 
         npixels = self.getStackOriginalImage().size
@@ -1103,17 +1107,17 @@ class StackBase(object):
                     (hasattr(motorValues, "ndim") and
                      motorValues.ndim == 0):
                 stackPositioners[motorName] = motorValues
-            elif hasattr(motorValues, "__len__") and numpy.isscalar(motorValues[0]):
-                # list: convert to numpy array before storing in info
-                numMotorValues = len(motorValues)
-                if numMotorValues == npixels:
-                    stackPositioners[motorName] = numpy.array(motorValues)
             elif hasattr(motorValues, "size"):
                 # numpy array
                 numMotorValues = motorValues.size
                 if numMotorValues == npixels:
                     stackPositioners[motorName] = numpy.array(motorValues,
                                                               copy=copy)
+            elif hasattr(motorValues, "__len__") and numpy.isscalar(motorValues[0]):
+                # list: convert to numpy array before storing in info
+                numMotorValues = len(motorValues)
+                if numMotorValues == npixels:
+                    stackPositioners[motorName] = numpy.array(motorValues)
             else:
                 raise TypeError(
                         "Wrong type for positioner %s. " % motorName +
