@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2019 V.A. Sole, ESRF - D. Dale CHESS
+# Copyright (C) 2004-2020 V.A. Sole, ESRF - D. Dale CHESS
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -37,6 +37,12 @@ from operator import itemgetter
 import logging
 _logger = logging.getLogger(__name__)
 
+if "hdf5plugin" not in sys.modules:
+    # try to import hdf5plugins
+    try:
+        import hdf5plugin
+    except:
+        _logger.info("Cannot import hdf5plugin")
 import h5py
 import weakref
 
@@ -68,14 +74,13 @@ else:
     def MyQVariant(x=None):
         return x
 
-
-
 QVERSION = qt.qVersion()
 
 
 #sorting method
-def h5py_sorting(object_list):
-    sorting_list = ['start_time', 'end_time', 'name']
+def h5py_sorting(object_list, sorting_list=None):
+    if sorting_list is None:
+        sorting_list = ['start_time', 'end_time', 'name']
     n = len(object_list)
     if n < 2:
         return object_list
@@ -111,6 +116,30 @@ def h5py_sorting(object_list):
             return object_list
 
     try:
+        if sorting_key == 'title':
+            def getTitle(x):
+                try:
+                    title = x["title"][()]
+                except:
+                    # allow the title to be missing
+                    title = ""
+                if hasattr(title, "dtype"):
+                    if hasattr(title, "__len__"):
+                        if len(title) == 1:
+                            title = title[0]
+                if hasattr(title, "decode"):
+                    title = title.decode("utf-8")
+                return title
+            # sort first by the traditional keys in order to be sorted
+            # by title and respecting actquisition order for equal title
+            try:
+                ordered_list = h5py_sorting(object_list)
+            except:
+                ordered_list = object_list
+            sorting_list = [(getTitle(o[1]), o) for o in ordered_list]
+            sorted_list = sorted(sorting_list, key=itemgetter(0))
+            return [x[1] for x in sorted_list]
+
         if sorting_key != 'name':
             sorting_list = [(o[1][sorting_key][()], o)
                            for o in object_list]
@@ -126,8 +155,8 @@ def h5py_sorting(object_list):
         #The only way to reach this point is to have different
         #structures among the different entries. In that case
         #defaults to the unfiltered case
-        print("WARNING: Default ordering")
-        print("Probably all entries do not have the key %s" % sorting_key)
+        _logger.warning("WARNING: Default ordering")
+        _logger.warning("Probably all entries do not have the key %s" % sorting_key)
         return object_list
 
 def _get_number_list(txt):
@@ -199,12 +228,21 @@ class RootItem(object):
         del self._identifiers[idx]
 
 class H5NodeProxy(object):
+    def sortChildren(self, column, order):
+        #print("sort children called with ", column, order)
+        self.__sorting = True
+        if column == 1:
+            self.__sorting_list = ["title"]
+        else:
+            self.__sorting_list = None
+        self.__sorting_order = order
+
     @property
     def children(self):
         if not self.hasChildren:
             return []
 
-        if not self._children:
+        if self.__sorting or not self._children:
             # obtaining the lock here is necessary, otherwise application can
             # freeze if navigating tree while data is processing
             if 1: #with self.file.plock:
@@ -216,7 +254,7 @@ class H5NodeProxy(object):
                     doit = False
                 try:
                     # better handling of external links
-                    finalList = h5py_sorting(items)
+                    finalList = h5py_sorting(items, sorting_list=self.__sorting_list)
                     for i in range(len(finalList)):
                         if finalList[i][1] is not None:
                             finalList[i][1]._posixPath = posixpath.join(self.name,
@@ -231,7 +269,7 @@ class H5NodeProxy(object):
                 except:
                     #one cannot afford any error, so I revert to the old
                     # method where values where used instead of items
-                    if _logger.getEffectiveLevel() == logging.DEBUG:
+                    if 1 or _logger.getEffectiveLevel() == logging.DEBUG:
                         raise
                     else:
                         # tmpList = list(self.getNode(self.name).values())
@@ -244,6 +282,7 @@ class H5NodeProxy(object):
                                                                items[i][0])
                     self._children = [H5NodeProxy(self.file, i, self)
                                       for i in finalList]
+        self.__sorting = False
         return self._children
 
     @property
@@ -300,6 +339,9 @@ class H5NodeProxy(object):
         return self._color
 
     def __init__(self, ffile, node, parent=None, path=None):
+        self.__sorting = False
+        self.__sorting_list = None
+        self.__sorting_order = qt.Qt.AscendingOrder
         if 1:#with ffile.plock:
             self._file = ffile
             self._parent = parent
@@ -357,7 +399,11 @@ class H5NodeProxy(object):
     def getNode(self, name=None):
         if not name:
             name = self.name
-        return self.file[name]
+        try:
+            return self.file[name]
+        except:
+            _logger.critical("Cannot access HDF5 file path <%s>" % name)
+            return name
 
     def __len__(self):
         return len(self.children)
@@ -400,6 +446,11 @@ class FileModel(qt.QAbstractItemModel):
         self.rootItem = RootItem(['File/Group/Dataset', 'Description', 'Shape', 'DType'])
         self._idMap = {qt.QModelIndex().internalId(): self.rootItem}
 
+    def sort(self, column, order):
+        #print("FileModel sort called with ", column, order)
+        for item in self.rootItem:
+            item.sortChildren(column, order)
+
     def clearRows(self, index):
         self.getProxyFromIndex(index).clearChildren()
 
@@ -429,17 +480,32 @@ class FileModel(qt.QAbstractItemModel):
                 if showtitle:
                     if hasattr(item, 'type'):
                         if item.type in ["Entry", "NXentry"]:
-                            children = item.children
-                            names = [posixpath.basename(o.name) for o in children]
-                            if "title" in names:
-                                idx = names.index("title")
-                                if len(children[idx].getNode().shape):
-                                    #stored as an array of strings!!!
-                                    #return just the first item
-                                    return MyQVariant("%s" % children[idx].getNode()[()][0])
-                                else:
-                                    #stored as a string
-                                    return MyQVariant("%s" % children[idx].getNode()[()])
+                            if hasattr(item, "children"):
+                                children = item.children
+                                names = [posixpath.basename(o.name) for o in children]
+                                if "title" in names:
+                                    idx = names.index("title")
+                                    node = children[idx].getNode()
+                                    if hasattr(node, "shape") and len(node.shape):
+                                        #stored as an array of strings???
+                                        #return just the first item
+                                        if hasattr(node, "asstr"):
+                                            try:
+                                                return MyQVariant("%s" % node.asstr()[()][0])
+                                            except:
+                                                return MyQVariant("%s" % node[()][0])
+                                    else:
+                                        #stored as a string
+                                        try:
+                                            try:
+                                                return MyQVariant("%s" % node.asstr()[()])
+                                            except:
+                                                return MyQVariant("%s" % node[()])
+                                        except:
+                                            # issue #745
+                                            return MyQVariant("Unknown %s" % node)
+                            else:
+                                _logger.critical("Entry %s has no children" % item.name)
                 return MyQVariant(item.type)
             if column == 2:
                 return MyQVariant(item.shape)
@@ -624,7 +690,6 @@ class FileView(qt.QTreeView):
             return
         self.fileUpdated(ddict)
 
-
     def fileUpdated(self, ddict):
         rootModelIndex = self.rootIndex()
         if self.model().hasChildren(rootModelIndex):
@@ -646,6 +711,8 @@ class HDF5Widget(FileView):
     def __init__(self, model, parent=None):
         FileView.__init__(self, model, parent)
         self.setSelectionBehavior(qt.QAbstractItemView.SelectRows)
+        self.setAutoScroll(False)
+
         self._adjust()
         if 0:
             self.activated[qt.QModelIndex].connect(self.itemActivated)
@@ -654,6 +721,49 @@ class HDF5Widget(FileView):
         self.doubleClicked[qt.QModelIndex].connect(self.itemDoubleClicked)
         self.collapsed[qt.QModelIndex].connect(self._adjust)
         self.expanded[qt.QModelIndex].connect(self._adjust)
+        self.setSortingEnabled(False)
+        self.header().sectionDoubleClicked[int].connect( \
+                         self._headerSectionDoubleClicked)
+        tip = "Double click on first two columns to change order"
+        self.header().setToolTip(tip)
+
+    def _headerSectionDoubleClicked(self, index):
+        self.sortItems(index, qt.Qt.AscendingOrder)
+
+    def __updateOrder(self):
+        rootModelIndex = self.rootIndex()
+        filelist = []
+        if self.model().hasChildren(rootModelIndex):
+            rootItem = self.model().getProxyFromIndex(rootModelIndex)
+            for row in range(len(rootItem)):
+                if self.model().hasIndex(row, 0, rootModelIndex):
+                    modelIndex = self.model().index(row, 0, rootModelIndex)
+                    item = self.model().getProxyFromIndex(modelIndex)
+                    try:
+                        filename = item.file.filename
+                        if filename not in filelist:
+                            filelist.append(filename)
+                    except:
+                        continue
+        if len(filelist):
+            for file in filelist:
+                ddict = {}
+                ddict['event'] = "fileUpdated"
+                ddict['filename'] = filename
+                self.fileUpdated(ddict)
+
+    def sortByColumn(self, column, order):
+        #reimplement QTreeWidget sorting
+        _logger.info("sort by column %d setting indicator %s" % (column, order))
+        self.setSortingEnabled(True)
+        self.header().setSortIndicator(column, order)
+        self.__updateOrder()
+        self.setSortingEnabled(False)
+
+    def sortItems(self, column, order):
+        #reimplement QTreeWidget sorting
+        _logger.info("sort items")
+        self.sortByColumn(column, order)
 
     def _adjust(self, modelIndex=None):
         self.resizeColumnToContents(0)
@@ -674,7 +784,7 @@ class HDF5Widget(FileView):
             self._lastMouse = "left"
         qt.QTreeView.mousePressEvent(self, e)
         if self._lastMouse != "left":
-            # Qt5 only sends itenClicked on left button mouse click
+            # Qt5 only sends itemClicked on left button mouse click
             if QVERSION > "5":
                 event = "itemClicked"
                 modelIndex = self.indexAt(e.pos())
@@ -724,16 +834,18 @@ class HDF5Widget(FileView):
         analyzedPaths = []
         for modelIndex in modelIndexList:
             item = self.model().getProxyFromIndex(modelIndex)
-            path = item.name * 1
-            if path in analyzedPaths:
-                continue
-            else:
-                analyzedPaths.append(path)
             if item.type in ["weakproxy", "File"]:
                 continue
+            filename = item.file.filename
+            path = item.name * 1
+            if (path, filename) in analyzedPaths:
+                continue
+            else:
+                analyzedPaths.append((path, filename))
             entry = "/" + path.split("/")[1]
-            if entry not in entryList:
-                entryList.append((entry, item.file.filename))
+            if (entry, filename) not in entryList:
+                entryList.append((entry, filename))
+        _logger.info("Returned entryList %s" % entryList)
         return entryList
 
 
