@@ -2,7 +2,7 @@
 #
 # The PyMca X-Ray Fluorescence Toolkit
 #
-# Copyright (c) 2004-2014 European Synchrotron Radiation Facility
+# Copyright (c) 2004-2020 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -26,7 +26,7 @@
 # THE SOFTWARE.
 #
 #############################################################################*/
-__author__ = "V. Armando Sole - ESRF Data Analysis"
+__author__ = "V. Armando Sole"
 __contact__ = "sole@esrf.fr"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
@@ -46,6 +46,7 @@ class Q3GridLayout(qt.QGridLayout):
 from PyMca5.PyMcaPhysics import Elements
 from . import MaterialEditor
 from . import MatrixEditor
+from . import TransmissionTableGui
 import re
 
 _logger = logging.getLogger(__name__)
@@ -68,9 +69,8 @@ class AttenuatorsTab(qt.QWidget):
         layout = qt.QVBoxLayout(self)
 
         maxheight = qt.QDesktopWidget().height()
-        if maxheight < 800:
-            layout.setContentsMargins(0, 0, 0, 0)
-            layout.setSpacing(2)
+        layout.setContentsMargins(0, 0, 0, 0)
+        layout.setSpacing(2)
 
         self.table  = AttenuatorsTableWidget(self, name, attenuators,
                                              funnyfilters=True)
@@ -86,10 +86,22 @@ class AttenuatorsTab(qt.QWidget):
         else:
             spacer = qt.VerticalSpacer(self)
             layout.addWidget(spacer)
-            self.editor = MaterialEditor.MaterialEditor(graph=graph)
-            self.table.setMinimumHeight(13*rheight)
+            if rheight > 32:
+                # when using big letters we run into troubles
+                # for instance windowx 1920x1080 but with a 150% scale
+                self.editor = MaterialEditor.MaterialEditor(height=5,
+                                                            graph=graph)
+                if rheight > 40:
+                    self.table.setMinimumHeight(10*rheight)
+                else:
+                    self.table.setMinimumHeight(13*rheight)
+            else:
+                self.editor = MaterialEditor.MaterialEditor(graph=graph)
+                self.table.setMinimumHeight(13*rheight)
             self.table.setMaximumHeight(13*rheight)
+        self.userAttenuators = TransmissionTableGui.TransmissionTableGui()
         self.mainTab.addTab(self.editor, "Material Editor")
+        self.mainTab.addTab(self.userAttenuators, "User Attenuators")
 
 class MultilayerTab(qt.QWidget):
     def __init__(self,parent=None, name="Multilayer Tab", matrixlayers=None):
