@@ -374,6 +374,8 @@ class MaskScatterWidget(PlotWindow.PlotWindow):
     def setSelectionMask(self, mask=None, plot=True):
         if self._selectionCurve is not None:
             selectionCurve = self.getCurve(self._selectionCurve)
+        else:
+            selectionCurve = None
         if selectionCurve in [[], None]:
             self._selectionCurve = None
             self._selectionMask = mask
@@ -482,8 +484,7 @@ class MaskScatterWidget(PlotWindow.PlotWindow):
             self.setZoomModeEnabled(True)
 
     def _rectSelectionIconSignal(self):
-        if DEBUG:
-            print("_rectSelectionIconSignal")
+        _logger.debug("_rectSelectionIconSignal")
         if self.rectSelectionToolButton.isChecked():
             self.setRectangularSelectionMode()
         else:
@@ -648,8 +649,7 @@ class MaskScatterWidget(PlotWindow.PlotWindow):
         self._brushMenu.exec_(self.cursor().pos())
 
     def _brushSelectionIconSignal(self):
-        if DEBUG:
-            print("_setBrushSelectionMode")
+        _logger.debug("_setBrushSelectionMode")
         if hasattr(self, "polygonSelectionToolButton"):
             self.polygonSelectionToolButton.setChecked(False)
             self.setDrawModeEnabled(False)
