@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #/*##########################################################################
-# Copyright (C) 2004-2020 V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2020 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -24,7 +24,7 @@
 # THE SOFTWARE.
 #
 #############################################################################*/
-__author__ = "V.A. Sole - ESRF Data Analysis"
+__author__ = "V.A. Sole"
 __contact__ = "sole@esrf.fr"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
@@ -37,12 +37,10 @@ import logging
 _logger = logging.getLogger(__name__)
 
 if __name__ == "__main__":
-    if "HDF5_USE_FILE_LOCKING" not in os.environ:
-        if "h5py" in sys.modules:
-             _logger.warning("h5py already imported")
-        os.environ["HDF5_USE_FILE_LOCKING"] = "FALSE"
-        _logger.info("%s set to %s" % ("HDF5_USE_FILE_LOCKING",
-                                       os.environ["HDF5_USE_FILE_LOCKING"]))
+    # We are going to read. Disable file locking.
+    os.environ["HDF5_USE_FILE_LOCKING"] = "FALSE"
+    _logger.info("%s set to %s" % ("HDF5_USE_FILE_LOCKING",
+                                    os.environ["HDF5_USE_FILE_LOCKING"]))
     try:
         # make sure hdf5plugins are imported
         import hdf5plugin
@@ -114,6 +112,7 @@ from PyMca5.PyMcaGui import PyMcaFileDialogs
 from PyMca5.PyMcaCore import DataObject
 from PyMca5.PyMcaGui.pymca import McaWindow
 from PyMca5.PyMcaCore import StackBase
+from PyMca5.PyMcaCore import McaStackExport
 from PyMca5.PyMcaGui import CloseEventNotifyingWidget
 from PyMca5.PyMcaGui import MaskImageWidget
 convertToRowAndColumn = MaskImageWidget.convertToRowAndColumn
@@ -190,6 +189,11 @@ class QStackWidget(StackBase.StackBase,
                                                     aspect=True)
         self._stackSaveMenu = qt.QMenu()
         if HDF5:
+            if self.master:
+                self._stackSaveMenu.addAction(QString("Export All Stacks Workspace"),
+                                                         self.exportStackList)
+            self._stackSaveMenu.addAction(QString("Export Stack Workspace"),
+                                                         self.exportStack)
             self._stackSaveMenu.addAction(QString("Save Zoomed Stack Region as Spectra"),
                                              self.saveStackAsNeXusSpectra)
             self._stackSaveMenu.addAction(QString("Save Zoomed Stack Region as Images"),
@@ -475,6 +479,28 @@ class QStackWidget(StackBase.StackBase,
             view = self._stack.data[row0:row1+1, col0:col1+1, :]
         return view
 
+    def exportStackList(self, filename=None):
+        if filename is None:
+            filename = self._getOutputHDF5Filename()
+            if not len(filename):
+                return
+            # the user already confirmed overwriting and McaStackExport does not
+            # delete an existing file
+            if os.path.exists(filename):
+                os.remove(filename)
+        McaStackExport.exportStackList(self.getStackDataObjectList(), filename)
+
+    def exportStack(self, filename=None):
+        if filename is None:
+            filename = self._getOutputHDF5Filename()
+            if not len(filename):
+                return
+            # the user already confirmed overwriting and McaStackExport does not
+            # delete an existing file
+            if os.path.exists(filename):
+                os.remove(filename)
+        McaStackExport.exportStackList([self.getStackDataObject()], filename)
+
     def saveStackAsNeXus(self, dtype=None, interpretation=None, compression=False):
         mcaIndex = self._stack.info.get('McaIndex', -1)
         if interpretation is None:
@@ -944,6 +970,10 @@ class QStackWidget(StackBase.StackBase,
 
         #ROI Image
         widgetList = [self.stackWidget, self.roiWidget]
+        if self.rgbWidget is not None:
+            if hasattr(self.rgbWidget, "sigMaskImageWidgetSignal"):
+                widgetList.append(self.rgbWidget)
+
         for widget in widgetList:
             widget.sigMaskImageWidgetSignal.connect(self._maskImageWidgetSlot)
 
@@ -1157,13 +1187,21 @@ class QStackWidget(StackBase.StackBase,
             _logger.debug("SLAVE setSelectionMask CALLED")
 
         #inform built in widgets
-        for widget in [self.stackWidget, self.roiWidget]:
+        widgetList = [self.stackWidget, self.roiWidget]
+        for widget in widgetList:
             if instance_id != id(widget):
                 if mask is None:
-                    widget._resetSelection(owncall=False)
+                    if hasattr(widget, "_resetSelection"):
+                        widget._resetSelection(owncall=False)
+                    else:
+                        widget.setSelectionMask(mask, plot=True)
                 else:
                     widget.setSelectionMask(mask, plot=True)
 
+        if self.rgbWidget is not None:
+            if hasattr(self.rgbWidget, "setSelectionMask"):
+                self.rgbWidget.setSelectionMask(mask, instance_id=instance_id)
+
         #inform slave
         if self._slaveList is not None:
             #This is a master instance
