@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2020 V.A. Sole, European Synchrotron Radiation Facility
+# Copyright (C) 2004-2020 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
 # the ESRF by the Software group.
@@ -68,6 +68,8 @@ except:
     NNMA = False
     PCA  = False
 
+KMEANS = RGBImageCalculator.KMEANS
+
 try:
     import tomogui.gui.utils.icons
     from tomogui.gui.ProjectWidget import ProjectWindow as TomoguiProjWindow
@@ -157,8 +159,9 @@ _logger = logging.getLogger(__name__)
 class RGBCorrelatorWidget(qt.QWidget):
 
     sigRGBCorrelatorWidgetSignal = qt.pyqtSignal(object)
+    sigMaskImageWidgetSignal = qt.pyqtSignal(object)
 
-    def __init__(self, parent = None, bgrx = False, replace = False):
+    def __init__(self, parent = None, bgrx = False, replace = False, image_shape=None):
         qt.QWidget.__init__(self, parent)
         self.replaceOption = replace
         self.setWindowTitle("RGBCorrelatorWidget")
@@ -277,7 +280,14 @@ class RGBCorrelatorWidget(qt.QWidget):
             self.bgrx = "RGBX"
         self._imageList = []
         self._imageDict = {}
-        self.__imageLength = None
+        if image_shape:
+            self.__imageLength = numpy.prod(image_shape, dtype=int)
+            self.__imageShape = tuple(image_shape)
+        else:
+            self.__imageLength = None
+            self.__imageShape = None
+        self.__imageLengthOriginal = self.__imageLength
+        self.__imageShapeOriginal = self.__imageShape
         self.__redLabel = None
         self.__greenLabel = None
         self.__blueLabel = None
@@ -308,7 +318,6 @@ class RGBCorrelatorWidget(qt.QWidget):
 
         self.toggleSlidersButton.clicked.connect(self.toggleSliders)
 
-        self.calculationButton.clicked.connect(self._showCalculationDialog)
         self.calculationButton.clicked.connect(self._showCalculationDialog)
 
         self.profileButton.clicked.connect(self.profileSelectedImages)
@@ -317,7 +326,9 @@ class RGBCorrelatorWidget(qt.QWidget):
         self.scatterPlotWidget = None
         self.pcaDialog  = None
         self.nnmaDialog = None
+        self.kMeansDialog = None
         self._tomoguiWindow = None
+        self._lastMask = None
 
         self.__imageResizeButton.clicked.connect(self._imageResizeSlot)
         self.sliderWidget.sigRGBCorrelatorSliderSignal.connect(self._sliderSlot)
@@ -327,7 +338,7 @@ class RGBCorrelatorWidget(qt.QWidget):
         self.buttonGroup.buttonClicked[int].connect(self._colormapTypeChange)
 
     def _showCalculationDialog(self):
-        if (not NNMA) and (not PCA):
+        if (not NNMA) and (not PCA) and (not KMEANS):
             return self.showCalculationDialog()
         if self._calculationMenu is None:
             self._calculationMenu = qt.QMenu()
@@ -345,6 +356,9 @@ class RGBCorrelatorWidget(qt.QWidget):
             if NNMA:
                 self._calculationMenu.addAction(QString("NNMA Analysis"),
                                             self.showNNMADialog)
+            if KMEANS:
+                self._calculationMenu.addAction(QString("K-Means Clustering"),
+                                            self.showKMeansDialog)
         self._calculationMenu.exec_(self.cursor().pos())
 
     def _showTomoReconsDialog(self):
@@ -688,7 +702,6 @@ class RGBCorrelatorWidget(qt.QWidget):
         self.addImageSlot(ddict)
 
     def _imageResizeSlot(self):
-        if self.__imageLength is None: return
         dialog = ImageShapeDialog(self, shape = self.__imageShape)
         dialog.setModal(True)
         ret = dialog.exec_()
@@ -709,11 +722,10 @@ class RGBCorrelatorWidget(qt.QWidget):
 
 
     def setImageShape(self, shape):
-        if self.__imageLength is None: return
-        length = 1
-        for value in shape:
-            length *= value
-        if length != self.__imageLength:
+        length = numpy.prod(shape, dtype=int)
+        if self.__imageLength is None:
+            self.__imageLength = length
+        elif length != self.__imageLength:
             raise ValueError("New length %d different of old length %d" % \
                     (length, self.__imageLength))
         self.__imageShape = shape
@@ -758,8 +770,8 @@ class RGBCorrelatorWidget(qt.QWidget):
         self._tableSlot({'r':[],'g':[],'b':[]})
         self._imageList = []
         self._imageDict = {}
-        self.__imageLength = None
-        self.__imageShape = None
+        self.__imageLength = self.__imageLengthOriginal
+        self.__imageShape = self.__imageShapeOriginal
         self.__redLabel    = None
         self.__greenLabel  = None
         self.__blueLabel   = None
@@ -1227,27 +1239,79 @@ class RGBCorrelatorWidget(qt.QWidget):
 
     def showCalculationDialog(self):
         if self.calculationDialog is None:
-            self.calculationDialog = RGBImageCalculator.RGBImageCalculator(replace=self.replaceOption)
+            selection = True
+            self.calculationDialog = RGBImageCalculator.RGBImageCalculator(\
+                                        replace=self.replaceOption,
+                                        selection=selection)
             self.calculationDialog.sigAddImageClicked.connect(self.addImageSlot)
             self.calculationDialog.sigRemoveImageClicked.connect( \
                          self.removeImage)
             if self.replaceOption:
                 self.calculationDialog.sigReplaceImageClicked.connect( \
                          self.replaceImageSlot)
+            if selection:
+                self.calculationDialog.graphWidget.sigMaskImageWidgetSignal.connect( \
+                                self.maskImageSlot)
+                self.calculationDialog.graphWidget.setSelectionMask( \
+                                    self.getSelectionMask())
         self.calculationDialog.imageList = self._imageList
         self.calculationDialog.imageDict = self._imageDict
         if self.calculationDialog.isHidden():
+            self.calculationDialog.graphWidget.setSelectionMask( \
+                                    self.getSelectionMask())
             self.calculationDialog.show()
         self.calculationDialog.raise_()
 
+    def showKMeansDialog(self):
+        itemList = self.tableWidget.selectedItems()
+        if len(itemList) < 1:
+            errorText = "Please select at least one image"
+            qt.QMessageBox.critical(self,"ValueError", errorText)
+            return
+        if self.kMeansDialog is None:
+            selection = True
+            self.kMeansDialog = RGBImageCalculator.RGBImageCalculator(math="kmeans",
+                                            replace=self.replaceOption,
+                                            selection=selection)
+            self.kMeansDialog.sigAddImageClicked.connect(self.addImageSlot)
+            self.kMeansDialog.sigRemoveImageClicked.connect( \
+                         self.removeImage)
+            if self.replaceOption:
+                self.kMeansDialog.sigReplaceImageClicked.connect( \
+                         self.replaceImageSlot)
+            if selection:
+                self.kMeansDialog.graphWidget.sigMaskImageWidgetSignal.connect( \
+                                self.maskImageSlot)
+                self.kMeansDialog.graphWidget.setSelectionMask( \
+                                    self.getSelectionMask())
+
+        # TODO: just pass a list or an array of images instead of a dict
+        nImages = len(self._imageList)
+        dataDict = {}
+        for item in itemList:
+            row = item.row()
+            if row >= nImages:
+                errorText = "Requested to work with non existing \nimage number %d." % row
+                qt.QMessageBox.critical(self,"ValueError", errorText)
+                return
+            label = self._imageList[row]
+            dataDict[label] = {"image": self._imageDict[label]['image']}
+        self.kMeansDialog.graphWidget.setImageData(None, clearmask=False)
+        self.kMeansDialog.imageDict = dataDict
+        if self.kMeansDialog.isHidden():
+            self.kMeansDialog.graphWidget.setSelectionMask( \
+                                self.getSelectionMask())
+            self.kMeansDialog.show()
+        self.kMeansDialog.raise_()
+
     def showScatterPlotDialog(self):
         if self.scatterPlotWidget is None:
             self.scatterPlotWidget = \
                 ScatterPlotCorrelatorWidget.ScatterPlotCorrelatorWidget(\
                                     labels=("Legend", "X", "Y"),
                                     types=("Text","RadioButton", "RadioButton"))
-        self.scatterPlotWidget.show()
-        self.scatterPlotWidget.raise_()
+            self.scatterPlotWidget.sigMaskScatterWidgetSignal.connect( \
+                                                      self.maskImageSlot)
         # I should check if the list is to be updated instead of systematically
         # send it
         initialize = True
@@ -1259,8 +1323,8 @@ class RGBCorrelatorWidget(qt.QWidget):
                 initialize = False
             else:
                 self.scatterPlotWidget.addSelectableItem(item, label=label)
-        if self.scatterPlotWidget.isHidden():
-            self.scatterPlotWidget.show()
+        self.scatterPlotWidget.setSelectionMask(self.getSelectionMask())
+        self.scatterPlotWidget.show()
         self.scatterPlotWidget.raise_()
 
     def getSelectedDataList(self):
@@ -1279,8 +1343,9 @@ class RGBCorrelatorWidget(qt.QWidget):
 
     def showPCADialog(self):
         if self.pcaDialog is None:
+            selection=True
             self.pcaDialog = PCADialog.PCADialog(rgbwidget=self,
-                                                 selection=False)
+                                                 selection=selection)
             self.pcaDialog.pcaWindow.\
                     buildAndConnectImageButtonBox(self.replaceOption)
             self.pcaDialog.pcaWindow.sigMaskImageWidgetSignal.connect(\
@@ -1293,13 +1358,15 @@ class RGBCorrelatorWidget(qt.QWidget):
             return
 
         self.pcaDialog.setData(datalist)
+        self.pcaDialog.pcaWindow.setSelectionMask(self.getSelectionMask())
         self.pcaDialog.show()
         self.pcaDialog.raise_()
 
     def showNNMADialog(self):
         if self.nnmaDialog is None:
+            selection=True
             self.nnmaDialog = NNMADialog.NNMADialog(rgbwidget=self,
-                                                 selection=False)
+                                                 selection=selection)
             self.nnmaDialog.nnmaWindow.\
                         buildAndConnectImageButtonBox(self.replaceOption)
             self.nnmaDialog.nnmaWindow.sigMaskImageWidgetSignal.connect(\
@@ -1312,6 +1379,7 @@ class RGBCorrelatorWidget(qt.QWidget):
             return
 
         self.nnmaDialog.setData(datalist)
+        self.nnmaDialog.nnmaWindow.setSelectionMask(self.getSelectionMask())
         self.nnmaDialog.show()
         self.nnmaDialog.raise_()
 
@@ -1328,7 +1396,32 @@ class RGBCorrelatorWidget(qt.QWidget):
             ddict['label'] = ddict['title']
             self.removeImageSlot(ddict)
             return
+        if ddict['event'] in ["selectionMaskChanged",
+                              "resetSelection",
+                              "invertSelection"]:
+            self.setSelectionMask(None, instance_id=ddict['id'])
+            self.sigMaskImageWidgetSignal.emit(ddict)
+            return
 
+    def setSelectionMask(self, mask, instance_id=None):
+        self._lastMask = mask
+        widgetList = []
+        if self.calculationDialog:
+            widgetList.append(self.calculationDialog.graphWidget)
+        if self.kMeansDialog:
+            widgetList.append(self.kMeansDialog.graphWidget)
+        if self.pcaDialog:
+            widgetList.append(self.pcaDialog.pcaWindow)
+        if self.nnmaDialog:
+            widgetList.append(self.nnmaDialog.nnmaWindow)
+        if self.scatterPlotWidget:
+            widgetList.append(self.scatterPlotWidget)
+        for widget in widgetList:
+            if id(widget) != instance_id:
+                widget.setSelectionMask(mask)
+
+    def getSelectionMask(self):
+        return self._lastMask
 
     def addImageSlot(self, ddict):
         self.addImage(ddict['image'], ddict['label'])
@@ -1431,13 +1524,19 @@ class ImageShapeDialog(qt.QDialog):
         self.okButton.clicked.connect(self.accept)
 
     def _rowsChanged(self):
+        if not self._size:
+            return
         nrows, ncolumns = self.getImageShape()
-        if (nrows * ncolumns) != self._size:
+        size = nrows * ncolumns
+        if size and size != self._size:
             self.columns.setText("%g" % (self._size/float(nrows)))
 
     def _columnsChanged(self):
+        if not self._size:
+            return
         nrows, ncolumns = self.getImageShape()
-        if (nrows * ncolumns) != self._size:
+        size = nrows * ncolumns
+        if size and size != self._size:
             self.rows.setText("%g" % (self._size/float(ncolumns)))
 
     def getImageShape(self):
